# Generated by Django 3.2.7 on 2022-07-28 05:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('purchase', '0005_alter_subscription_user'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='_is_active',
            field=models.DateField(null=True),
        ),
    ]



from django.db import models

from purchase.models.choices import Platform


class AppsFlyer(models.Model):
    app_id = models.CharField(max_length=100)
    dev_key = models.CharField(max_length=300)
    platform = models.CharField(choices=Platform.choices, max_length=20, unique=True)

    class Meta:
        verbose_name_plural = "AppsFlyer"



import time

from dataclasses import dataclass
from typing import List, Dict

from purchase.utils import repeatable_request
from purchase.strings.verifiers import APPLE_URL, APPLE_SANDBOX_URL, APPLE_HEADERS


@dataclass
class AppleVerifier:
    receipt: dict
    is_sandbox: bool
    product_id: str
    platform: str
    version: str
    transaction_id: str
    is_sub: bool = False

    def get_url(self):
        if self.is_sandbox:
            return APPLE_SANDBOX_URL
        return APPLE_URL

    def verify(self) -> (bool, bool):
        data = {"receipt-data": self.receipt["payload"]}
        url = self.get_url()
        is_sandbox, result = self.request(url, data)
        return is_sandbox, result

    def get_pos(self, products: List[Dict]) -> int:
        pos = [
            ind
            for ind, product in enumerate(products)
            if product.get("product_id") == self.product_id
            and product.get("transaction_id") == self.transaction_id
        ]
        if len(pos):
            return pos[0]
        pos = [
            ind
            for ind, product in enumerate(products)
            if product.get("original_transaction_id") == self.transaction_id
        ]
        return pos[0]

    def products_is_ok(self, products: List[Dict]) -> bool:
        contains = (
            any(
                True
                for product in products
                if product.get("product_id") == self.product_id
                and product.get("transaction_id") == self.transaction_id
            ),
            any(
                True
                for product in products
                if product.get("original_transaction_id") == self.transaction_id
            )
        )
        if not self.is_sub:
            return any(contains)
        pos = self.get_pos(products)
        product = products[pos]
        exp = product.get("expires_date_ms")
        now = round(time.time() * 1000)
        return now < exp

    def request(self, url: str, data: dict) -> (bool, bool):
        is_sandbox = self.is_sandbox
        result = False

        response = repeatable_request(url=url, json=data, headers=APPLE_HEADERS).json()

        if response["status"] == 21008:  # is not sandbox
            is_sandbox = False
            response = repeatable_request(
                url=APPLE_URL, json=data, headers=APPLE_HEADERS
            ).json()

        if response["status"] == 21007:  # this is sandbox
            is_sandbox = True
            response = repeatable_request(
                url=APPLE_SANDBOX_URL, json=data, headers=APPLE_HEADERS
            ).json()

        if response["status"] == 0:
            products = response["receipt"]["in_app"]
            result = self.products_is_ok(products)

        return is_sandbox, result



from django.db import models

from purchase.models.choices import Platform


class Log(models.Model):
    """Error logs when making purchases"""

    version = models.CharField(max_length=10, null=True)
    log_level = models.CharField(max_length=15, null=True, blank=True)
    platform = models.CharField(choices=Platform.choices, max_length=20)

    message = models.TextField()

    details = models.JSONField(null=True, blank=True)

    time = models.DateTimeField(auto_now_add=True)



import json
from datetime import datetime
from functools import lru_cache

from django import forms
from django.db import IntegrityError
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.urls import re_path, reverse
from django.utils.html import format_html
from django.contrib import admin, messages
from django.core.handlers.wsgi import WSGIRequest
from django.utils import timezone
from django.utils.safestring import mark_safe
from pygments import highlight
from pygments.formatters.html import HtmlFormatter
from pygments.lexers.data import JsonLexer
from rangefilter.filters import DateTimeRangeFilter
from django.core.exceptions import ValidationError

from purchase.models import (
    Config,
    Log,
    Purchase,
    AdjustLog,
    Facebook,
    Adjust,
    AppsFlyer,
    Google,
)


class FileUploadForm(forms.Form):
    file = forms.FileField()

    def replace_model(self, data):
        if isinstance(data, dict):
            if data["model"] == Google._meta.label_lower:
                self.data = data["fields"]
            else:
                self.add_error(
                    None,
                    f'JSON parse error: field model must be {Google._meta.label_lower}, found {data["model"]}',
                )
        else:
            self.add_error(None, "JSON parse error: root object must be <dict>")

    def clean_file(self):
        raw_data = self.cleaned_data["file"]
        for chunk in raw_data.chunks():
            data = json.loads(chunk)
            self.replace_model(data)
        return raw_data

    def save(self, instance):
        if self.data:
            instance.update(**self.data)
            return instance


class CustomDateTimeRangeFilter(DateTimeRangeFilter):
    def queryset(self, request: WSGIRequest, queryset):
        if self.form.is_valid():
            log_time_gte = str(
                int(self.form.cleaned_data["log_time__range__gte"].timestamp())
            )
            log_time_lte = str(
                int(self.form.cleaned_data["log_time__range__lte"].timestamp())
            )
            if log_time_gte and log_time_lte:
                return queryset.filter(log_time__range=(log_time_gte, log_time_lte))
        return queryset


@lru_cache()
def get_fb_credentials(platform):
    return Facebook.objects.filter(platform=platform).first()


def pretty_printed(obj, attr_name):
    """Function to display pretty version of our data"""
    # Convert the data to sorted, indented JSON
    response = json.dumps(getattr(obj, attr_name), sort_keys=True, indent=4)
    # Truncate the data. Alter as needed
    # response = response[:5000]  # noqa: E800
    # Get the Pygments formatter
    formatter = HtmlFormatter(style="colorful")
    # Highlight the data
    response = highlight(response, JsonLexer(), formatter)
    # Get the stylesheet
    style = "<style>" + formatter.get_style_defs() + "</style><br>"
    # Safe the output
    return mark_safe(style + response)  # noqa: S308, S703


@admin.register(Purchase)
class PurchaseAdmin(admin.ModelAdmin):
    readonly_fields = ("pretty_printed",)
    list_display = (
        "transaction_id",
        "version",
        "platform",
        "product_id",
        "value_to_sum",
        "currency",
        "time",
        "is_valid",
        "fb_is_logged",
        "af_is_logged",
        "adjust_is_logged",
        "is_sandbox",
    )
    list_display_links = ("transaction_id",)
    list_filter = (
        "platform",
        "is_valid",
        "fb_is_logged",
        "af_is_logged",
        "adjust_is_logged",
        "bundle_short_version",
        ("log_time", CustomDateTimeRangeFilter),
    )

    def get_rangefilter_logTime_default(self, _request):  # noqa: N802
        return timezone.datetime.today, timezone.datetime.today

    def get_rangefilter_logTime_title(self, _request, _field_path):  # noqa: N802
        return "Date Filter"

    def time(self, obj):
        return datetime.fromtimestamp(int(obj.log_time)).strftime("%Y-%m-%d %H:%M:%S")

    time.short_description = "Time"

    def version(self, obj):
        return obj.bundle_short_version

    version.short_description = "Version"

    def pretty_printed(self, obj):
        return pretty_printed(obj, "body")

    pretty_printed.short_description = "Pretty printed"


@admin.register(Config)
class ConfigAdmin(admin.ModelAdmin):
    list_display = (
        "platform",
        "is_fb_log_enabled",
        "is_af_log_enabled",
        "is_adjust_log_enabled",
    )

    actions = [
        "enable_facebook_logging",
        "disable_facebook_logging",
        "enable_appsflyer_logging",
        "disable_appsflyer_logging",
        "enable_adjust_logging",
        "disable_adjust_logging",
    ]

    def disable_adjust_logging(self, request, queryset):
        self.message_user(
            request,
            "Adjust logging was successfully disabled.",
            messages.SUCCESS,
        )

    def disable_appsflyer_logging(self, request, queryset):
        self.message_user(
            request,
            "AppsFlyer logging was successfully disabled.",
            messages.SUCCESS,
        )

    def disable_facebook_logging(self, request, queryset):
        self.message_user(
            request,
            "Facebook logging was successfully disabled.",
            messages.SUCCESS,
        )

    def enable_adjust_logging(self, request, queryset):
        self.message_user(
            request,
            "Adjust logging was successfully enabled.",
            messages.SUCCESS,
        )

    def enable_appsflyer_logging(self, request, queryset):
        self.message_user(
            request,
            "AppsFlyer logging was successfully enabled.",
            messages.SUCCESS,
        )

    def enable_facebook_logging(self, request, queryset):
        self.message_user(
            request,
            "Facebook logging was successfully enabled.",
            messages.SUCCESS,
        )


@admin.register(Log)
class LogAdmin(admin.ModelAdmin):
    readonly_fields = ("pretty_details",)
    list_display = (
        "log_time",
        "platform",
        "version",
        "log_level",
        "message",
    )
    list_filter = (
        ("time", DateTimeRangeFilter),
        "platform",
        "log_level",
        "message",
        "version",
    )

    def get_rangefilter_time_default(self, _request):
        return timezone.datetime.today, timezone.datetime.today

    def get_rangefilter_time_title(self, _request, _field_path):
        return "Date Filter"

    @staticmethod
    def log_time(obj):
        return obj.time.strftime("%Y-%m-%d %H:%M:%S")

    @staticmethod
    def pretty_details(obj):
        return pretty_printed(obj, "details")

    pretty_details.short_description = "details"


@admin.register(AdjustLog)
class AdjustLogAdmin(admin.ModelAdmin):
    list_display = (
        "get_transaction_id",
        "get_platform",
        "get_version",
        "request_data",
        "response",
        "created_at",
    )
    list_filter = (
        ("created_at", DateTimeRangeFilter),
        "purchase__platform",
        "purchase__bundle_short_version",
    )
    search_fields = ("purchase__transaction_id",)
    fieldsets = (
        (None, {"fields": ("purchase",)}),
        (
            "Request",
            {"fields": ("request_url", "request_headers_pp", "request_data_pp")},
        ),
        ("Response", {"fields": ("response_pp",)}),
    )

    readonly_fields = (
        "purchase",
        "request_headers_pp",
        "request_data_pp",
        "request_url",
        "response_pp",
    )

    def get_rangefilter_time_default(self, _request):
        return timezone.datetime.today, timezone.datetime.today

    def get_rangefilter_time_title(self, _request, _field_path):
        return "Date Filter"

    def get_platform(self, obj):
        return obj.purchase.platform.title()

    get_platform.short_description = "Platform"
    get_platform.admin_order_field = "purchase__platform"

    def get_version(self, obj):
        return obj.purchase.bundle_short_version

    get_version.short_description = "Version"
    get_version.admin_order_field = "purchase__bundle_short_version"

    def get_transaction_id(self, obj):
        return obj.purchase.transaction_id

    get_transaction_id.short_description = "Transaction id"
    get_transaction_id.admin_order_field = "purchase__transaction_id"

    def request_headers_pp(self, obj):
        return pretty_printed(obj, "request_headers")

    request_headers_pp.short_description = "Headers"

    def request_data_pp(self, obj):
        return pretty_printed(obj, "request_data")

    request_data_pp.short_description = "Body"

    def response_pp(self, obj):
        return pretty_printed(obj, "response")

    response_pp.short_description = "Data"


@admin.register(AppsFlyer)
class AppsFlyerAdmin(admin.ModelAdmin):
    list_display = ("platform", "app_id", "dev_key")
    fieldsets = (
        (None, {"fields": ("platform", )}),
        ("App", {"fields": ("app_id", "dev_key")}),
    )


@admin.register(Facebook)
class FacebookAdmin(admin.ModelAdmin):
    list_display = ("platform", "app_id", "client_secret")

    fieldsets = (
        (None, {"fields": ("platform", )}),
        ("App", {"fields": ("app_id", "client_secret")}),
    )


@admin.register(Google)
class GoogleAdmin(admin.ModelAdmin):
    list_display = (
        "type",
        "project_id",
        "json_action",
        "client_email",
    )
    list_filter = ("client_email",)

    fieldsets = (
        (None, {"fields": ("type", "project_id")}),
        ("keys", {"fields": ("private_key_id", "private_key")}),
        ("client credentials", {"fields": ("client_email", "client_id")}),
        ("auth uri", {"fields": ("auth_uri", "token_uri")}),
        ("certs", {"fields": ("auth_provider_x509_cert_url", "client_x509_cert_url")}),
    )

    def __init__(self, model, admin_site):
        super().__init__(model, admin_site)
        self.json_action_form = FileUploadForm

    def load_from_json(self, request, instance_id):
        return self.process_action(
            request=request,
            instance_id=instance_id,
            action_title=None,
        )

    def get_urls(self):
        urls = super().get_urls()
        custom_urls = [
            re_path(
                r"^(?P<instance_id>.+)/load-from-json/$",
                self.admin_site.admin_view(self.load_from_json),
                name="load-from-json",
            ),
        ]
        return custom_urls + urls

    def json_action(self, obj):
        return format_html(
            '<a class="button" href="{}">JSON</a>',
            reverse("admin:load-from-json", args=[obj.pk]),
        )

    json_action.short_description = "Load Values"
    json_action.allow_tags = True

    def process_action(self, request, instance_id, action_title):
        queryset = self.get_queryset(request).filter(pk=instance_id)
        if request.method != "POST":
            form = self.json_action_form()
        else:
            form, redirect = self.form_with_file_request(request, queryset, instance_id)
            if redirect is not None:
                return redirect
        context = self.admin_site.each_context(request)
        context["opts"] = self.model._meta
        context["form"] = form
        context["google"] = queryset
        context["title"] = action_title
        return TemplateResponse(
            request,
            "google_json_load_form.html",
            context,
        )

    def form_with_file_request(self, request, queryset, instance_id):
        form = self.json_action_form(request.POST, request.FILES)
        if form.is_valid():
            redirect_url = (
                f"/djadmin/{Google._meta.app_label}/{Google._meta.model_name}/"
            )
            try:
                form.save(queryset)
            except (ValidationError, IntegrityError) as e:
                messages.error(request, f"Database error: {e}")
                return form, HttpResponseRedirect(redirect_url)
            messages.success(request, "Success")
            return form, HttpResponseRedirect(f"{redirect_url}{instance_id}/")
        return (form, )


@admin.register(Adjust)
class AdjustAdmin(admin.ModelAdmin):
    list_display = (
        "platform",
        "app_token",
        "purchase_event_token",
        "authorization_token",
    )



# Generated by Django 3.2.7 on 2022-08-01 08:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('purchase', '0008_auto_20220801_0606'),
    ]

    operations = [
        migrations.AlterField(
            model_name='purchase',
            name='bundle_short_version',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='bundle_short_version',
            field=models.CharField(max_length=255, null=True),
        ),
    ]



from django.db import models
from django.forms.models import model_to_dict


class Google(models.Model):
    client_id = models.CharField(max_length=50)
    project_id = models.CharField(max_length=50)
    private_key_id = models.CharField(max_length=60)
    type = models.CharField(max_length=20, default="service_account")

    private_key = models.TextField()

    client_email = models.EmailField()

    auth_uri = models.URLField()
    token_uri = models.URLField()
    client_x509_cert_url = models.URLField()
    auth_provider_x509_cert_url = models.URLField()

    class Meta:
        verbose_name_plural = "Google"

    @property
    def as_dict(self):  # pragma: no cover
        return model_to_dict(
            self,
            exclude=["id", "game"],
        )



import logging

from django.contrib.auth.models import AnonymousUser
from rest_framework.response import Response
from rest_framework.parsers import JSONParser
from rest_framework.generics import CreateAPIView

from purchase.exceptions import CustomStatus

logger = logging.getLogger(__name__)


class ProcessTransactionBaseView(CreateAPIView):
    parser_classes = [JSONParser]

    def post(self, request, *args, **kwargs):
        request_data = self.request_serializer(data=request.data)
        try:
            request_data.is_valid(raise_exception=True)
        except Exception as err:
            logger.log(logging.ERROR, str(err))
            return Response(data=self.get_response_data(self.status_choices.data_is_not_valid), status=200)
        request_data = request_data.validated_data
        self.update_data(request_data)
        try:
            response = self.process(data=request_data)
            return response
        except CustomStatus as cs:
            return Response(data=self.get_response_data(cs.message), status=200)
        except Exception as err:
            logger.log(logging.ERROR, err)
            return Response(data=self.get_response_data(self.status_choices.error), status=200)

    @property
    def status_choices(self):
        raise NotImplementedError

    @property
    def controller_class(self):
        raise NotImplementedError

    @property
    def signal(self):
        raise NotImplementedError

    @property
    def request_serializer(self):
        raise NotImplementedError

    @property
    def response_serializer(self):
        raise NotImplementedError

    @property
    def use_user(self):
        raise NotImplementedError

    def update_data(self, request_data: dict):
        if hasattr(self.request, "user") and not isinstance(self.request.user, AnonymousUser) and self.use_user:
            request_data.update({"user": self.request.user})
        else:
            request_data.update({"user": None})

    def process(self, data: dict):
        purchase = self.controller_class(serializer_data=data)

        if purchase.is_create:
            raise CustomStatus(self.status_choices.already_created)

        create_is_done, purchase_model = purchase.try_to_create()
        if not create_is_done:
            raise CustomStatus(self.status_choices.data_is_not_valid)

        is_sandbox, is_valid = purchase.verify()
        if is_sandbox:
            response_data = self.get_response_data(status=self.status_choices.ok)
            return Response(data=response_data, status=200)

        if is_valid:
            if hasattr(purchase, "lc"):
                purchase.lc.log(purchase_obj=purchase_model)
            purchase_model.is_valid = True
            purchase_model.save()
            response_data = self.get_response_data(status=self.status_choices.ok)
        else:
            purchase_model.set_transaction_id_to_fake()
            response_data = self.get_response_data(status=self.status_choices.not_valid)

        if self.use_user:
            self.signal.send(sender=self.__class__, instance=purchase)

        return Response(data=response_data, status=200)

    def get_response_data(self, status: str, error: str = None):
        serializable_data = {"status": status}
        if error:
            serializable_data.update({"error": error})
        response_data = self.response_serializer(data=serializable_data)
        response_data.is_valid(raise_exception=True)
        return response_data.validated_data



import json
import logging
from abc import ABC, abstractmethod

from django.db import transaction

from purchase.controllers import LogController
from purchase.models.choices import Platform
from purchase.strings.log_levels import PURCHASE_CREATE

logger = logging.getLogger(__name__)


class ProcessControllerABC(ABC):
    def __init__(self, serializer_data: dict):
        self.serializer_data = serializer_data
        self.platform = serializer_data["platform"]
        self.fb = serializer_data["data"]["fb"] if "fb" in serializer_data["data"] else None
        self.version = \
            serializer_data["data"]["bundle_short_version"] if "bundle_short_version" in serializer_data["data"] \
            else None
        self.is_sandbox = serializer_data["is_sandbox"]
        self.receipt_data = serializer_data["data"]["receipt_data"]
        self.user = serializer_data.pop("user")
        if self.logging_enabled:
            self.lc = LogController(self.platform, self.version, self.fb, self.transaction_id, self.serializer_data)

    @property
    @abstractmethod
    def model(self):
        raise NotImplementedError

    @property
    @abstractmethod
    def logging_enabled(self):
        raise NotImplementedError

    @property
    def transaction_id(self):
        if self.platform == Platform.android:
            return self.get_transaction_id_from_json()
        else:
            return self.serializer_data["data"]["receipt_data"]["transaction_id"]

    def get_transaction_id_from_json(self):
        try:
            payload = json.loads(self.receipt_data["payload"])
            payload_json = json.loads(payload["json"])
            return payload_json["orderId"]
        except Exception as err:
            logger.error(
                f"JSON parsing of transaction_id from payload on Android: {err}"
            )

    @transaction.atomic
    def create(self):
        return self.model.objects.create(
            user=self.user,
            transaction_id=self.transaction_id,
            advertiser_id=self.serializer_data["data"]["advertiser_id"],
            platform=self.platform,
            fb_user_id=self.fb["user_id"] if self.fb else None,
            bundle_short_version=self.version,
            ext_info=self.fb["extinfo"] if self.fb else None,
            product_id=self.serializer_data["data"]["product_id"],
            value_to_sum=self.serializer_data["data"]["value_to_sum"],
            log_time=self.serializer_data["data"]["log_time"],
            currency=self.serializer_data["data"]["currency"],
            is_sandbox=self.is_sandbox,
            body=self.serializer_data
        )

    @property
    def is_create(self):
        return self.model.objects.filter(
            transaction_id=self.transaction_id, platform=self.platform
        ).exists()

    def try_to_create(self) -> (bool, model or bool):
        try:
            purchase_obj = self.create()
            return True, purchase_obj
        except Exception as err:
            if self.logging_enabled:
                self.lc.save_error_log(
                    error_message=str(err),
                    log_level=PURCHASE_CREATE,
                    details=self.serializer_data,
                )
            logger.error(str(err))
            return False, False

    def verify(self) -> (bool, bool):
        if self.platform == Platform.android:
            return self.google_verify()
        return self.apple_verify()

    @abstractmethod
    def google_verify(self):
        raise NotImplementedError

    @abstractmethod
    def apple_verify(self):
        raise NotImplementedError



# Generated by Django 3.2.7 on 2022-07-27 08:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('purchase', '0003_auto_20220215_0611'),
    ]

    operations = [
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user', models.UUIDField()),
                ('currency', models.CharField(max_length=25)),
                ('log_time', models.CharField(max_length=11)),
                ('ext_info', models.CharField(max_length=255)),
                ('product_id', models.CharField(max_length=255)),
                ('transaction_id', models.CharField(max_length=255)),
                ('bundle_short_version', models.CharField(max_length=255)),
                ('fb_user_id', models.CharField(max_length=255, null=True)),
                ('advertiser_id', models.CharField(max_length=255, null=True)),
                ('platform', models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=8)),
                ('value_to_sum', models.FloatField()),
                ('body', models.JSONField()),
                ('is_sandbox', models.BooleanField(default=False)),
                ('active_till', models.DateTimeField(null=True)),
                ('google_token', models.CharField(help_text='Only for android platform', max_length=8192, null=True)),
                ('_is_active', models.DateField()),
                ('_expired', models.BooleanField(default=False)),
            ],
        ),
    ]



import json
import logging
import requests

from datetime import datetime
from dataclasses import dataclass
from django.shortcuts import get_object_or_404

from purchase.models import AppsFlyer
from purchase.utils import repeatable_request

logger = logging.getLogger(__name__)


@dataclass
class AppsFlyerLogger:
    platform: str
    appsflyer_id: str
    fb: dict
    advertiser_id: str
    bundle_short_version: str
    log_time: int
    currency: str
    value_to_sum: float
    product_id: str

    def log_purchase(self) -> requests.Response:
        response = repeatable_request(
            url=self._get_url(), json=self._get_data(), headers=self._get_headers()
        )
        return response

    @property
    def _credentials(self):
        credentials = get_object_or_404(
            AppsFlyer, platform=self.platform
        )
        return credentials

    @property
    def _time_format(self):
        return "%Y-%m-%d %H-%M-%S.000"

    def _get_data(self):
        data = {
            "appsflyer_id": self.appsflyer_id,
            "advertising_id": self.advertiser_id,
            "app_version_name": self.bundle_short_version,
            "eventTime": datetime.fromtimestamp(self.log_time).strftime(
                self._time_format
            ),
            "eventName": "af_purchase",
            "eventCurrency": self.currency,
            "eventValue": json.dumps(
                {
                    "af_revenue": self.value_to_sum,
                    "af_content_type": "In-App Purchase",
                    "af_content_id": self.product_id,
                    "af_quantity": 1,
                }
            ),
        }

        if len(self.fb["extinfo"]) >= 2:
            data["bundleIdentifier"] = self.fb["extinfo"][1]

        return data

    def _get_headers(self):
        return {"authentication": self._credentials.dev_key}

    def _get_url(self):
        return f"https://api2.appsflyer.com/inappevent/{self._credentials.app_id}"



import time
import logging
import requests

from typing import Union


logger = logging.getLogger(__name__)


def repeatable_request(
    url: str,
    params: dict = None,
    data: Union[str, dict] = None,
    json: dict = None,
    headers: dict = None,
    trying: int = 3,
) -> requests.Response:
    response = request(url, params, data, json, headers)
    while trying and response.status_code != 200:
        trying -= 1
        time.sleep(0.1)

    if response.status_code != 200:
        error_text = f"api error: request error [{url}] {response.status_code = }, {response.text = }"
        logger.exception(error_text)
    return response


def request(url, params, data, json, headers):
    response = requests.post(
        url=url, params=params, data=data, json=json, headers=headers
    )
    response_text = response.text  # for debugging in sentry  # noqa: F841
    return response



from django.urls import path

from .views import ProcessPurchaseView, LogConfig, ProcessSubscriptionView

urlpatterns = [
    path("chargeverify/", ProcessPurchaseView.as_view(), name="ChargeVerify"),
    path("subverify/", ProcessSubscriptionView.as_view(), name="SubscriptionVerify"),
    path("logconfig/", LogConfig.as_view(), name="LogConfig"),
]



# Generated by Django 3.2.7 on 2022-02-15 06:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('purchase', '0002_auto_20210920_1019'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='google',
            name='unique',
        ),
        migrations.RemoveField(
            model_name='adjust',
            name='game',
        ),
        migrations.RemoveField(
            model_name='appsflyer',
            name='game',
        ),
        migrations.RemoveField(
            model_name='config',
            name='game',
        ),
        migrations.RemoveField(
            model_name='facebook',
            name='game',
        ),
        migrations.RemoveField(
            model_name='google',
            name='game',
        ),
        migrations.RemoveField(
            model_name='log',
            name='game',
        ),
        migrations.RemoveField(
            model_name='purchase',
            name='game',
        ),
        migrations.AlterField(
            model_name='adjust',
            name='platform',
            field=models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=20, unique=True),
        ),
        migrations.AlterField(
            model_name='appsflyer',
            name='platform',
            field=models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=20, unique=True),
        ),
        migrations.AlterField(
            model_name='config',
            name='platform',
            field=models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=20, unique=True),
        ),
        migrations.AlterField(
            model_name='facebook',
            name='platform',
            field=models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=20, unique=True),
        ),
    ]



from django.db import models

from purchase.models.choices import Platform


class Adjust(models.Model):
    app_token = models.CharField(max_length=50)
    authorization_token = models.CharField(max_length=50)
    purchase_event_token = models.CharField(max_length=50)
    platform = models.CharField(choices=Platform.choices, max_length=20, unique=True)

    class Meta:
        verbose_name_plural = "Adjust"



import json
import logging

from dataclasses import dataclass

from google.oauth2 import service_account
from google.auth.transport.requests import AuthorizedSession

from purchase.models import Google as GoogleModel
from purchase.strings.verifiers import GOOGLE_AUTH_URL

logger = logging.getLogger(__name__)


@dataclass
class GoogleVerifier:
    receipt: dict
    platform: str
    version: str
    is_sub: bool = False

    @property
    def payload(self) -> dict:
        payload = self.receipt["payload"]
        if isinstance(payload, str):
            payload = json.loads(payload)
        return json.loads(payload["json"])

    @property
    def url(self) -> str:
        package_name = self.payload["packageName"]
        product_id = self.payload["productId"]
        token = self.payload["purchaseToken"]
        base_url = "https://www.googleapis.com/androidpublisher/v3/applications/"
        url_type = "products" if not self.is_sub else "subscriptions"
        data_url = f"{package_name}/purchases/{url_type}/{product_id}/tokens/{token}"
        return base_url + data_url

    def verify(self) -> bool:
        associated_google_model = GoogleModel.objects.first()
        if not associated_google_model:
            return False
        instance = associated_google_model
        kwargs = instance.as_dict
        kwargs["private_key"] = kwargs["private_key"].replace("\\n", "\n")
        credentials = service_account.Credentials.from_service_account_info(kwargs)
        credentials = credentials.with_scopes([GOOGLE_AUTH_URL])
        session = AuthorizedSession(credentials)
        response = session.get(self.url)
        return response.status_code == 200



from django.db import models

from purchase.models.choices import Platform


class Config(models.Model):
    """Model for setting purchase logging settings"""

    platform = models.CharField(choices=Platform.choices, max_length=20, unique=True)

    is_fb_log_enabled = models.BooleanField(default=True)
    is_af_log_enabled = models.BooleanField(default=True)
    is_adjust_log_enabled = models.BooleanField(default=True)



from typing import Union
from rest_framework import status


class CustomStatus(Exception):
    status_code = status.HTTP_200_OK
    message = "Status"
    details = ""

    def __init__(
        self,
        message: Union[str, int] = None,
        details: str = None,
    ):
        self.message = message or self.message
        self.details = details or self.details

    def __str__(self):
        details = f": {self.details}" if self.details else ""
        return f"{self.message}{details}"



from .log_controller import LogController
from .process_controller_abc import ProcessControllerABC
from .purchase_controller import PurchaseProcessController
from .subscription_controller import SubscriptionController



from django.db import models


class AdjustLog(models.Model):
    """Adjust logs"""

    purchase = models.OneToOneField(
        to="purchase.Purchase", on_delete=models.CASCADE, related_name="adjust_log"
    )

    request_url = models.URLField()

    response = models.JSONField()
    request_data = models.JSONField()
    request_headers = models.JSONField()

    created_at = models.DateTimeField(auto_now=True)



from .process_transaction_base_view import ProcessTransactionBaseView
from .process_purchase_view import ProcessPurchaseView
from .process_subscription_view import ProcessSubscriptionView
from .log_config import LogConfig
